package java.util;

import java.util.concurrent.CountedCompleter;

class ArraysParallelSortHelpers
{
    ArraysParallelSortHelpers() {
    }
    
    static final class EmptyCompleter extends CountedCompleter<Void>
    {
        static final long serialVersionUID = 2446542900576103244L;
        
        EmptyCompleter(CountedCompleter<?> completer) {
            super(completer);
        }
        
        @Override
        public final void compute() {
        }
    }
    
    static final class Relay extends CountedCompleter<Void>
    {
        static final long serialVersionUID = 2446542900576103244L;
        final CountedCompleter<?> task;
        
        Relay(CountedCompleter<?> task) {
            super(null, 1);
            this.task = task;
        }
        
        @Override
        public final void compute() {
        }
        
        @Override
        public final void onCompletion(CountedCompleter<?> countedCompleter) {
            this.task.compute();
        }
    }
    
    static final class FJObject
    {
        static final class Sorter<T> extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final T[] a;
            final T[] w;
            final int base;
            final int size;
            final int wbase;
            final int gran;
            Comparator<? super T> comparator;
            
            Sorter(CountedCompleter<?> completer, T[] a, T[] w, int base, int size, int wbase, int gran, Comparator<? super T> comparator) {
                super(completer);
                this.a = a;
                this.w = w;
                this.base = base;
                this.size = size;
                this.wbase = wbase;
                this.gran = gran;
                this.comparator = comparator;
            }
            
            @Override
            public final void compute() {
                Comparator<Object> comparator = this.comparator;
                Object[] a = this.a;
                Object[] w = this.w;
                int base = this.base;
                int i = this.size;
                int wbase = this.wbase;
                int n2;
                Object o3;
                for (int gran = this.gran; i > gran; this = new ArraysParallelSortHelpers.EmptyCompleter(o3), i = n2) {
                    int n = i >>> 1;
                    n2 = n >>> 1;
                    int n3 = n + n2;
                    ArraysParallelSortHelpers.Relay o = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJObject.Merger(this, w, a, wbase, n, wbase + n, i - n, base, gran, comparator));
                    ArraysParallelSortHelpers.Relay o2 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJObject.Merger(o, a, w, base + n, n2, base + n3, i - n3, wbase + n, gran, comparator));
                    new ArraysParallelSortHelpers.FJObject.Sorter(o2, a, w, base + n3, i - n3, wbase + n3, gran, comparator).fork();
                    new ArraysParallelSortHelpers.FJObject.Sorter(o2, a, w, base + n, n2, wbase + n, gran, comparator).fork();
                    o3 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJObject.Merger(o, a, w, base, n2, base + n2, n - n2, wbase, gran, comparator));
                    new ArraysParallelSortHelpers.FJObject.Sorter(o3, a, w, base + n2, n - n2, wbase + n2, gran, comparator).fork();
                }
                TimSort.sort(a, base, base + i, comparator, w, wbase, i);
                this.tryComplete();
            }
        }
        
        static final class Merger<T> extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final T[] a;
            final T[] w;
            final int lbase;
            final int lsize;
            final int rbase;
            final int rsize;
            final int wbase;
            final int gran;
            Comparator<? super T> comparator;
            
            Merger(CountedCompleter<?> completer, T[] a, T[] w, int lbase, int lsize, int rbase, int rsize, int wbase, int gran, Comparator<? super T> comparator) {
                super(completer);
                this.a = a;
                this.w = w;
                this.lbase = lbase;
                this.lsize = lsize;
                this.rbase = rbase;
                this.rsize = rsize;
                this.wbase = wbase;
                this.gran = gran;
                this.comparator = comparator;
            }
            
            @Override
            public final void compute() {
                Comparator<Object> comparator = this.comparator;
                Object[] a = this.a;
                Object[] w = this.w;
                int lbase = this.lbase;
                int lsize = this.lsize;
                int rbase = this.rbase;
                int rsize = this.rsize;
                int wbase = this.wbase;
                int gran = this.gran;
                if (a == null || w == null || lbase < 0 || rbase < 0 || wbase < 0 || comparator == null) {
                    throw new IllegalStateException();
                }
                while (true) {
                    int n;
                    int n2;
                    if (lsize >= rsize) {
                        if (lsize <= gran) {
                            break;
                        }
                        n2 = rsize;
                        Object o2 = a[(n = lsize >>> 1) + lbase];
                        int j = 0;
                        while (j < n2) {
                            int n4 = j + n2 >>> 1;
                            if (comparator.compare(o2, a[n4 + rbase]) <= 0) {
                                n2 = n4;
                            }
                            else {
                                j = n4 + 1;
                            }
                        }
                    }
                    else {
                        if (rsize <= gran) {
                            break;
                        }
                        n = lsize;
                        Object o = a[(n2 = rsize >>> 1) + rbase];
                        int i = 0;
                        while (i < n) {
                            int n3 = i + n >>> 1;
                            if (comparator.compare(o, a[n3 + lbase]) <= 0) {
                                n = n3;
                            }
                            else {
                                i = n3 + 1;
                            }
                        }
                    }
                    ArraysParallelSortHelpers.FJObject.Merger o3 = new ArraysParallelSortHelpers.FJObject.Merger(this, a, w, lbase + n, lsize - n, rbase + n2, rsize - n2, wbase + n + n2, gran, comparator);
                    rsize = n2;
                    lsize = n;
                    this.addToPendingCount(1);
                    o3.fork();
                }
                int n5;
                int n6;
                Object o6;
                for (n5 = lbase + lsize, n6 = rbase + rsize; lbase < n5 && rbase < n6; w[wbase++] = o6) {
                    Comparator<Object> comparator2 = comparator;
                    Object o4 = a[lbase];
                    Object o5;
                    if (comparator2.compare(o4, o5 = a[rbase]) <= 0) {
                        ++lbase;
                        o6 = o4;
                    }
                    else {
                        ++rbase;
                        o6 = o5;
                    }
                }
                if (rbase < n6) {
                    System.arraycopy(a, rbase, w, wbase, n6 - rbase);
                }
                else if (lbase < n5) {
                    System.arraycopy(a, lbase, w, wbase, n5 - lbase);
                }
                this.tryComplete();
                return;
            }
        }
    }
    
    static final class FJByte
    {
        static final class Sorter extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final byte[] a;
            final byte[] w;
            final int base;
            final int size;
            final int wbase;
            final int gran;
            
            Sorter(CountedCompleter<?> completer, byte[] a, byte[] w, int base, int size, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.base = base;
                this.size = size;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                byte[] a = this.a;
                byte[] w = this.w;
                int base = this.base;
                int i = this.size;
                int wbase = this.wbase;
                int n2;
                Object o3;
                for (int gran = this.gran; i > gran; this = new ArraysParallelSortHelpers.EmptyCompleter(o3), i = n2) {
                    int n = i >>> 1;
                    n2 = n >>> 1;
                    int n3 = n + n2;
                    ArraysParallelSortHelpers.Relay o = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJByte.Merger(this, w, a, wbase, n, wbase + n, i - n, base, gran));
                    ArraysParallelSortHelpers.Relay o2 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJByte.Merger(o, a, w, base + n, n2, base + n3, i - n3, wbase + n, gran));
                    new ArraysParallelSortHelpers.FJByte.Sorter(o2, a, w, base + n3, i - n3, wbase + n3, gran).fork();
                    new ArraysParallelSortHelpers.FJByte.Sorter(o2, a, w, base + n, n2, wbase + n, gran).fork();
                    o3 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJByte.Merger(o, a, w, base, n2, base + n2, n - n2, wbase, gran));
                    new ArraysParallelSortHelpers.FJByte.Sorter(o3, a, w, base + n2, n - n2, wbase + n2, gran).fork();
                }
                DualPivotQuicksort.sort(a, base, base + i - 1);
                this.tryComplete();
            }
        }
        
        static final class Merger extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final byte[] a;
            final byte[] w;
            final int lbase;
            final int lsize;
            final int rbase;
            final int rsize;
            final int wbase;
            final int gran;
            
            Merger(CountedCompleter<?> completer, byte[] a, byte[] w, int lbase, int lsize, int rbase, int rsize, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.lbase = lbase;
                this.lsize = lsize;
                this.rbase = rbase;
                this.rsize = rsize;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                byte[] a = this.a;
                byte[] w = this.w;
                int lbase = this.lbase;
                int lsize = this.lsize;
                int rbase = this.rbase;
                int rsize = this.rsize;
                int wbase = this.wbase;
                int gran = this.gran;
                if (a == null || w == null || lbase < 0 || rbase < 0 || wbase < 0) {
                    throw new IllegalStateException();
                }
                while (true) {
                    int n;
                    int n3;
                    if (lsize >= rsize) {
                        if (lsize <= gran) {
                            break;
                        }
                        n3 = rsize;
                        int n5 = a[(n = lsize >>> 1) + lbase];
                        int j = 0;
                        while (j < n3) {
                            int n6 = j + n3 >>> 1;
                            if (n5 <= a[n6 + rbase]) {
                                n3 = n6;
                            }
                            else {
                                j = n6 + 1;
                            }
                        }
                    }
                    else {
                        if (rsize <= gran) {
                            break;
                        }
                        n = lsize;
                        int n2 = a[(n3 = rsize >>> 1) + rbase];
                        int i = 0;
                        while (i < n) {
                            int n4 = i + n >>> 1;
                            if (n2 <= a[n4 + lbase]) {
                                n = n4;
                            }
                            else {
                                i = n4 + 1;
                            }
                        }
                    }
                    ArraysParallelSortHelpers.FJByte.Merger o = new ArraysParallelSortHelpers.FJByte.Merger(this, a, w, lbase + n, lsize - n, rbase + n3, rsize - n3, wbase + n + n3, gran);
                    rsize = n3;
                    lsize = n;
                    this.addToPendingCount(1);
                    o.fork();
                }
                int n7;
                int n8;
                int n11;
                for (n7 = lbase + lsize, n8 = rbase + rsize; lbase < n7 && rbase < n8; w[wbase++] = n11) {
                    int n9 = a[lbase];
                    int n10;
                    if (n9 <= (n10 = a[rbase])) {
                        ++lbase;
                        n11 = n9;
                    }
                    else {
                        ++rbase;
                        n11 = n10;
                    }
                }
                if (rbase < n8) {
                    System.arraycopy(a, rbase, w, wbase, n8 - rbase);
                }
                else if (lbase < n7) {
                    System.arraycopy(a, lbase, w, wbase, n7 - lbase);
                }
                this.tryComplete();
                return;
            }
        }
    }
    
    static final class FJChar
    {
        static final class Sorter extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final char[] a;
            final char[] w;
            final int base;
            final int size;
            final int wbase;
            final int gran;
            
            Sorter(CountedCompleter<?> completer, char[] a, char[] w, int base, int size, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.base = base;
                this.size = size;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                char[] a = this.a;
                char[] w = this.w;
                int base = this.base;
                int i = this.size;
                int wbase = this.wbase;
                int n2;
                Object o3;
                for (int gran = this.gran; i > gran; this = new ArraysParallelSortHelpers.EmptyCompleter(o3), i = n2) {
                    int n = i >>> 1;
                    n2 = n >>> 1;
                    int n3 = n + n2;
                    ArraysParallelSortHelpers.Relay o = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJChar.Merger(this, w, a, wbase, n, wbase + n, i - n, base, gran));
                    ArraysParallelSortHelpers.Relay o2 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJChar.Merger(o, a, w, base + n, n2, base + n3, i - n3, wbase + n, gran));
                    new ArraysParallelSortHelpers.FJChar.Sorter(o2, a, w, base + n3, i - n3, wbase + n3, gran).fork();
                    new ArraysParallelSortHelpers.FJChar.Sorter(o2, a, w, base + n, n2, wbase + n, gran).fork();
                    o3 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJChar.Merger(o, a, w, base, n2, base + n2, n - n2, wbase, gran));
                    new ArraysParallelSortHelpers.FJChar.Sorter(o3, a, w, base + n2, n - n2, wbase + n2, gran).fork();
                }
                DualPivotQuicksort.sort(a, base, base + i - 1, w, wbase, i);
                this.tryComplete();
            }
        }
        
        static final class Merger extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final char[] a;
            final char[] w;
            final int lbase;
            final int lsize;
            final int rbase;
            final int rsize;
            final int wbase;
            final int gran;
            
            Merger(CountedCompleter<?> completer, char[] a, char[] w, int lbase, int lsize, int rbase, int rsize, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.lbase = lbase;
                this.lsize = lsize;
                this.rbase = rbase;
                this.rsize = rsize;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                char[] a = this.a;
                char[] w = this.w;
                int lbase = this.lbase;
                int lsize = this.lsize;
                int rbase = this.rbase;
                int rsize = this.rsize;
                int wbase = this.wbase;
                int gran = this.gran;
                if (a == null || w == null || lbase < 0 || rbase < 0 || wbase < 0) {
                    throw new IllegalStateException();
                }
                while (true) {
                    int n;
                    int n3;
                    if (lsize >= rsize) {
                        if (lsize <= gran) {
                            break;
                        }
                        n3 = rsize;
                        int n5 = a[(n = lsize >>> 1) + lbase];
                        int j = 0;
                        while (j < n3) {
                            int n6 = j + n3 >>> 1;
                            if (n5 <= a[n6 + rbase]) {
                                n3 = n6;
                            }
                            else {
                                j = n6 + 1;
                            }
                        }
                    }
                    else {
                        if (rsize <= gran) {
                            break;
                        }
                        n = lsize;
                        int n2 = a[(n3 = rsize >>> 1) + rbase];
                        int i = 0;
                        while (i < n) {
                            int n4 = i + n >>> 1;
                            if (n2 <= a[n4 + lbase]) {
                                n = n4;
                            }
                            else {
                                i = n4 + 1;
                            }
                        }
                    }
                    ArraysParallelSortHelpers.FJChar.Merger o = new ArraysParallelSortHelpers.FJChar.Merger(this, a, w, lbase + n, lsize - n, rbase + n3, rsize - n3, wbase + n + n3, gran);
                    rsize = n3;
                    lsize = n;
                    this.addToPendingCount(1);
                    o.fork();
                }
                int n7;
                int n8;
                int n11;
                for (n7 = lbase + lsize, n8 = rbase + rsize; lbase < n7 && rbase < n8; w[wbase++] = n11) {
                    int n9 = a[lbase];
                    int n10;
                    if (n9 <= (n10 = a[rbase])) {
                        ++lbase;
                        n11 = n9;
                    }
                    else {
                        ++rbase;
                        n11 = n10;
                    }
                }
                if (rbase < n8) {
                    System.arraycopy(a, rbase, w, wbase, n8 - rbase);
                }
                else if (lbase < n7) {
                    System.arraycopy(a, lbase, w, wbase, n7 - lbase);
                }
                this.tryComplete();
                return;
            }
        }
    }
    
    static final class FJShort
    {
        static final class Sorter extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final short[] a;
            final short[] w;
            final int base;
            final int size;
            final int wbase;
            final int gran;
            
            Sorter(CountedCompleter<?> completer, short[] a, short[] w, int base, int size, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.base = base;
                this.size = size;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                short[] a = this.a;
                short[] w = this.w;
                int base = this.base;
                int i = this.size;
                int wbase = this.wbase;
                int n2;
                Object o3;
                for (int gran = this.gran; i > gran; this = new ArraysParallelSortHelpers.EmptyCompleter(o3), i = n2) {
                    int n = i >>> 1;
                    n2 = n >>> 1;
                    int n3 = n + n2;
                    ArraysParallelSortHelpers.Relay o = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJShort.Merger(this, w, a, wbase, n, wbase + n, i - n, base, gran));
                    ArraysParallelSortHelpers.Relay o2 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJShort.Merger(o, a, w, base + n, n2, base + n3, i - n3, wbase + n, gran));
                    new ArraysParallelSortHelpers.FJShort.Sorter(o2, a, w, base + n3, i - n3, wbase + n3, gran).fork();
                    new ArraysParallelSortHelpers.FJShort.Sorter(o2, a, w, base + n, n2, wbase + n, gran).fork();
                    o3 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJShort.Merger(o, a, w, base, n2, base + n2, n - n2, wbase, gran));
                    new ArraysParallelSortHelpers.FJShort.Sorter(o3, a, w, base + n2, n - n2, wbase + n2, gran).fork();
                }
                DualPivotQuicksort.sort(a, base, base + i - 1, w, wbase, i);
                this.tryComplete();
            }
        }
        
        static final class Merger extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final short[] a;
            final short[] w;
            final int lbase;
            final int lsize;
            final int rbase;
            final int rsize;
            final int wbase;
            final int gran;
            
            Merger(CountedCompleter<?> completer, short[] a, short[] w, int lbase, int lsize, int rbase, int rsize, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.lbase = lbase;
                this.lsize = lsize;
                this.rbase = rbase;
                this.rsize = rsize;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                short[] a = this.a;
                short[] w = this.w;
                int lbase = this.lbase;
                int lsize = this.lsize;
                int rbase = this.rbase;
                int rsize = this.rsize;
                int wbase = this.wbase;
                int gran = this.gran;
                if (a == null || w == null || lbase < 0 || rbase < 0 || wbase < 0) {
                    throw new IllegalStateException();
                }
                while (true) {
                    int n;
                    int n3;
                    if (lsize >= rsize) {
                        if (lsize <= gran) {
                            break;
                        }
                        n3 = rsize;
                        int n5 = a[(n = lsize >>> 1) + lbase];
                        int j = 0;
                        while (j < n3) {
                            int n6 = j + n3 >>> 1;
                            if (n5 <= a[n6 + rbase]) {
                                n3 = n6;
                            }
                            else {
                                j = n6 + 1;
                            }
                        }
                    }
                    else {
                        if (rsize <= gran) {
                            break;
                        }
                        n = lsize;
                        int n2 = a[(n3 = rsize >>> 1) + rbase];
                        int i = 0;
                        while (i < n) {
                            int n4 = i + n >>> 1;
                            if (n2 <= a[n4 + lbase]) {
                                n = n4;
                            }
                            else {
                                i = n4 + 1;
                            }
                        }
                    }
                    ArraysParallelSortHelpers.FJShort.Merger o = new ArraysParallelSortHelpers.FJShort.Merger(this, a, w, lbase + n, lsize - n, rbase + n3, rsize - n3, wbase + n + n3, gran);
                    rsize = n3;
                    lsize = n;
                    this.addToPendingCount(1);
                    o.fork();
                }
                int n7;
                int n8;
                int n11;
                for (n7 = lbase + lsize, n8 = rbase + rsize; lbase < n7 && rbase < n8; w[wbase++] = n11) {
                    int n9 = a[lbase];
                    int n10;
                    if (n9 <= (n10 = a[rbase])) {
                        ++lbase;
                        n11 = n9;
                    }
                    else {
                        ++rbase;
                        n11 = n10;
                    }
                }
                if (rbase < n8) {
                    System.arraycopy(a, rbase, w, wbase, n8 - rbase);
                }
                else if (lbase < n7) {
                    System.arraycopy(a, lbase, w, wbase, n7 - lbase);
                }
                this.tryComplete();
                return;
            }
        }
    }
    
    static final class FJInt
    {
        static final class Sorter extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final int[] a;
            final int[] w;
            final int base;
            final int size;
            final int wbase;
            final int gran;
            
            Sorter(CountedCompleter<?> completer, int[] a, int[] w, int base, int size, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.base = base;
                this.size = size;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                int[] a = this.a;
                int[] w = this.w;
                int base = this.base;
                int i = this.size;
                int wbase = this.wbase;
                int n2;
                Object o3;
                for (int gran = this.gran; i > gran; this = new ArraysParallelSortHelpers.EmptyCompleter(o3), i = n2) {
                    int n = i >>> 1;
                    n2 = n >>> 1;
                    int n3 = n + n2;
                    ArraysParallelSortHelpers.Relay o = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJInt.Merger(this, w, a, wbase, n, wbase + n, i - n, base, gran));
                    ArraysParallelSortHelpers.Relay o2 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJInt.Merger(o, a, w, base + n, n2, base + n3, i - n3, wbase + n, gran));
                    new ArraysParallelSortHelpers.FJInt.Sorter(o2, a, w, base + n3, i - n3, wbase + n3, gran).fork();
                    new ArraysParallelSortHelpers.FJInt.Sorter(o2, a, w, base + n, n2, wbase + n, gran).fork();
                    o3 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJInt.Merger(o, a, w, base, n2, base + n2, n - n2, wbase, gran));
                    new ArraysParallelSortHelpers.FJInt.Sorter(o3, a, w, base + n2, n - n2, wbase + n2, gran).fork();
                }
                DualPivotQuicksort.sort(a, base, base + i - 1, w, wbase, i);
                this.tryComplete();
            }
        }
        
        static final class Merger extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final int[] a;
            final int[] w;
            final int lbase;
            final int lsize;
            final int rbase;
            final int rsize;
            final int wbase;
            final int gran;
            
            Merger(CountedCompleter<?> completer, int[] a, int[] w, int lbase, int lsize, int rbase, int rsize, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.lbase = lbase;
                this.lsize = lsize;
                this.rbase = rbase;
                this.rsize = rsize;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                int[] a = this.a;
                int[] w = this.w;
                int lbase = this.lbase;
                int lsize = this.lsize;
                int rbase = this.rbase;
                int rsize = this.rsize;
                int wbase = this.wbase;
                int gran = this.gran;
                if (a == null || w == null || lbase < 0 || rbase < 0 || wbase < 0) {
                    throw new IllegalStateException();
                }
                while (true) {
                    int n;
                    int n3;
                    if (lsize >= rsize) {
                        if (lsize <= gran) {
                            break;
                        }
                        n3 = rsize;
                        int n5 = a[(n = lsize >>> 1) + lbase];
                        int j = 0;
                        while (j < n3) {
                            int n6 = j + n3 >>> 1;
                            if (n5 <= a[n6 + rbase]) {
                                n3 = n6;
                            }
                            else {
                                j = n6 + 1;
                            }
                        }
                    }
                    else {
                        if (rsize <= gran) {
                            break;
                        }
                        n = lsize;
                        int n2 = a[(n3 = rsize >>> 1) + rbase];
                        int i = 0;
                        while (i < n) {
                            int n4 = i + n >>> 1;
                            if (n2 <= a[n4 + lbase]) {
                                n = n4;
                            }
                            else {
                                i = n4 + 1;
                            }
                        }
                    }
                    ArraysParallelSortHelpers.FJInt.Merger o = new ArraysParallelSortHelpers.FJInt.Merger(this, a, w, lbase + n, lsize - n, rbase + n3, rsize - n3, wbase + n + n3, gran);
                    rsize = n3;
                    lsize = n;
                    this.addToPendingCount(1);
                    o.fork();
                }
                int n7;
                int n8;
                int n11;
                for (n7 = lbase + lsize, n8 = rbase + rsize; lbase < n7 && rbase < n8; w[wbase++] = n11) {
                    int n9 = a[lbase];
                    int n10;
                    if (n9 <= (n10 = a[rbase])) {
                        ++lbase;
                        n11 = n9;
                    }
                    else {
                        ++rbase;
                        n11 = n10;
                    }
                }
                if (rbase < n8) {
                    System.arraycopy(a, rbase, w, wbase, n8 - rbase);
                }
                else if (lbase < n7) {
                    System.arraycopy(a, lbase, w, wbase, n7 - lbase);
                }
                this.tryComplete();
                return;
            }
        }
    }
    
    static final class FJLong
    {
        static final class Sorter extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final long[] a;
            final long[] w;
            final int base;
            final int size;
            final int wbase;
            final int gran;
            
            Sorter(CountedCompleter<?> completer, long[] a, long[] w, int base, int size, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.base = base;
                this.size = size;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                long[] a = this.a;
                long[] w = this.w;
                int base = this.base;
                int i = this.size;
                int wbase = this.wbase;
                int n2;
                Object o3;
                for (int gran = this.gran; i > gran; this = new ArraysParallelSortHelpers.EmptyCompleter(o3), i = n2) {
                    int n = i >>> 1;
                    n2 = n >>> 1;
                    int n3 = n + n2;
                    ArraysParallelSortHelpers.Relay o = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJLong.Merger(this, w, a, wbase, n, wbase + n, i - n, base, gran));
                    ArraysParallelSortHelpers.Relay o2 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJLong.Merger(o, a, w, base + n, n2, base + n3, i - n3, wbase + n, gran));
                    new ArraysParallelSortHelpers.FJLong.Sorter(o2, a, w, base + n3, i - n3, wbase + n3, gran).fork();
                    new ArraysParallelSortHelpers.FJLong.Sorter(o2, a, w, base + n, n2, wbase + n, gran).fork();
                    o3 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJLong.Merger(o, a, w, base, n2, base + n2, n - n2, wbase, gran));
                    new ArraysParallelSortHelpers.FJLong.Sorter(o3, a, w, base + n2, n - n2, wbase + n2, gran).fork();
                }
                DualPivotQuicksort.sort(a, base, base + i - 1, w, wbase, i);
                this.tryComplete();
            }
        }
        
        static final class Merger extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final long[] a;
            final long[] w;
            final int lbase;
            final int lsize;
            final int rbase;
            final int rsize;
            final int wbase;
            final int gran;
            
            Merger(CountedCompleter<?> completer, long[] a, long[] w, int lbase, int lsize, int rbase, int rsize, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.lbase = lbase;
                this.lsize = lsize;
                this.rbase = rbase;
                this.rsize = rsize;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                long[] a = this.a;
                long[] w = this.w;
                int lbase = this.lbase;
                int lsize = this.lsize;
                int rbase = this.rbase;
                int rsize = this.rsize;
                int wbase = this.wbase;
                int gran = this.gran;
                if (a == null || w == null || lbase < 0 || rbase < 0 || wbase < 0) {
                    throw new IllegalStateException();
                }
                while (true) {
                    int n;
                    int n3;
                    if (lsize >= rsize) {
                        if (lsize <= gran) {
                            break;
                        }
                        n3 = rsize;
                        long n5 = a[(n = lsize >>> 1) + lbase];
                        int j = 0;
                        while (j < n3) {
                            int n6 = j + n3 >>> 1;
                            if (n5 <= a[n6 + rbase]) {
                                n3 = n6;
                            }
                            else {
                                j = n6 + 1;
                            }
                        }
                    }
                    else {
                        if (rsize <= gran) {
                            break;
                        }
                        n = lsize;
                        long n2 = a[(n3 = rsize >>> 1) + rbase];
                        int i = 0;
                        while (i < n) {
                            int n4 = i + n >>> 1;
                            if (n2 <= a[n4 + lbase]) {
                                n = n4;
                            }
                            else {
                                i = n4 + 1;
                            }
                        }
                    }
                    ArraysParallelSortHelpers.FJLong.Merger o = new ArraysParallelSortHelpers.FJLong.Merger(this, a, w, lbase + n, lsize - n, rbase + n3, rsize - n3, wbase + n + n3, gran);
                    rsize = n3;
                    lsize = n;
                    this.addToPendingCount(1);
                    o.fork();
                }
                int n7;
                int n8;
                long n11;
                for (n7 = lbase + lsize, n8 = rbase + rsize; lbase < n7 && rbase < n8; w[wbase++] = n11) {
                    long n9 = a[lbase];
                    long n10;
                    if (n9 <= (n10 = a[rbase])) {
                        ++lbase;
                        n11 = n9;
                    }
                    else {
                        ++rbase;
                        n11 = n10;
                    }
                }
                if (rbase < n8) {
                    System.arraycopy(a, rbase, w, wbase, n8 - rbase);
                }
                else if (lbase < n7) {
                    System.arraycopy(a, lbase, w, wbase, n7 - lbase);
                }
                this.tryComplete();
                return;
            }
        }
    }
    
    static final class FJFloat
    {
        static final class Sorter extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final float[] a;
            final float[] w;
            final int base;
            final int size;
            final int wbase;
            final int gran;
            
            Sorter(CountedCompleter<?> completer, float[] a, float[] w, int base, int size, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.base = base;
                this.size = size;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                float[] a = this.a;
                float[] w = this.w;
                int base = this.base;
                int i = this.size;
                int wbase = this.wbase;
                int n2;
                Object o3;
                for (int gran = this.gran; i > gran; this = new ArraysParallelSortHelpers.EmptyCompleter(o3), i = n2) {
                    int n = i >>> 1;
                    n2 = n >>> 1;
                    int n3 = n + n2;
                    ArraysParallelSortHelpers.Relay o = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJFloat.Merger(this, w, a, wbase, n, wbase + n, i - n, base, gran));
                    ArraysParallelSortHelpers.Relay o2 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJFloat.Merger(o, a, w, base + n, n2, base + n3, i - n3, wbase + n, gran));
                    new ArraysParallelSortHelpers.FJFloat.Sorter(o2, a, w, base + n3, i - n3, wbase + n3, gran).fork();
                    new ArraysParallelSortHelpers.FJFloat.Sorter(o2, a, w, base + n, n2, wbase + n, gran).fork();
                    o3 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJFloat.Merger(o, a, w, base, n2, base + n2, n - n2, wbase, gran));
                    new ArraysParallelSortHelpers.FJFloat.Sorter(o3, a, w, base + n2, n - n2, wbase + n2, gran).fork();
                }
                DualPivotQuicksort.sort(a, base, base + i - 1, w, wbase, i);
                this.tryComplete();
            }
        }
        
        static final class Merger extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final float[] a;
            final float[] w;
            final int lbase;
            final int lsize;
            final int rbase;
            final int rsize;
            final int wbase;
            final int gran;
            
            Merger(CountedCompleter<?> completer, float[] a, float[] w, int lbase, int lsize, int rbase, int rsize, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.lbase = lbase;
                this.lsize = lsize;
                this.rbase = rbase;
                this.rsize = rsize;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                float[] a = this.a;
                float[] w = this.w;
                int lbase = this.lbase;
                int lsize = this.lsize;
                int rbase = this.rbase;
                int rsize = this.rsize;
                int wbase = this.wbase;
                int gran = this.gran;
                if (a == null || w == null || lbase < 0 || rbase < 0 || wbase < 0) {
                    throw new IllegalStateException();
                }
                while (true) {
                    int n;
                    int n3;
                    if (lsize >= rsize) {
                        if (lsize <= gran) {
                            break;
                        }
                        n3 = rsize;
                        float n5 = a[(n = lsize >>> 1) + lbase];
                        int j = 0;
                        while (j < n3) {
                            int n6 = j + n3 >>> 1;
                            if (n5 <= a[n6 + rbase]) {
                                n3 = n6;
                            }
                            else {
                                j = n6 + 1;
                            }
                        }
                    }
                    else {
                        if (rsize <= gran) {
                            break;
                        }
                        n = lsize;
                        float n2 = a[(n3 = rsize >>> 1) + rbase];
                        int i = 0;
                        while (i < n) {
                            int n4 = i + n >>> 1;
                            if (n2 <= a[n4 + lbase]) {
                                n = n4;
                            }
                            else {
                                i = n4 + 1;
                            }
                        }
                    }
                    ArraysParallelSortHelpers.FJFloat.Merger o = new ArraysParallelSortHelpers.FJFloat.Merger(this, a, w, lbase + n, lsize - n, rbase + n3, rsize - n3, wbase + n + n3, gran);
                    rsize = n3;
                    lsize = n;
                    this.addToPendingCount(1);
                    o.fork();
                }
                int n7;
                int n8;
                float n11;
                for (n7 = lbase + lsize, n8 = rbase + rsize; lbase < n7 && rbase < n8; w[wbase++] = n11) {
                    float n9 = a[lbase];
                    float n10;
                    if (n9 <= (n10 = a[rbase])) {
                        ++lbase;
                        n11 = n9;
                    }
                    else {
                        ++rbase;
                        n11 = n10;
                    }
                }
                if (rbase < n8) {
                    System.arraycopy(a, rbase, w, wbase, n8 - rbase);
                }
                else if (lbase < n7) {
                    System.arraycopy(a, lbase, w, wbase, n7 - lbase);
                }
                this.tryComplete();
                return;
            }
        }
    }
    
    static final class FJDouble
    {
        static final class Sorter extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final double[] a;
            final double[] w;
            final int base;
            final int size;
            final int wbase;
            final int gran;
            
            Sorter(CountedCompleter<?> completer, double[] a, double[] w, int base, int size, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.base = base;
                this.size = size;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                double[] a = this.a;
                double[] w = this.w;
                int base = this.base;
                int i = this.size;
                int wbase = this.wbase;
                int n2;
                Object o3;
                for (int gran = this.gran; i > gran; this = new ArraysParallelSortHelpers.EmptyCompleter(o3), i = n2) {
                    int n = i >>> 1;
                    n2 = n >>> 1;
                    int n3 = n + n2;
                    ArraysParallelSortHelpers.Relay o = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJDouble.Merger(this, w, a, wbase, n, wbase + n, i - n, base, gran));
                    ArraysParallelSortHelpers.Relay o2 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJDouble.Merger(o, a, w, base + n, n2, base + n3, i - n3, wbase + n, gran));
                    new ArraysParallelSortHelpers.FJDouble.Sorter(o2, a, w, base + n3, i - n3, wbase + n3, gran).fork();
                    new ArraysParallelSortHelpers.FJDouble.Sorter(o2, a, w, base + n, n2, wbase + n, gran).fork();
                    o3 = new ArraysParallelSortHelpers.Relay(new ArraysParallelSortHelpers.FJDouble.Merger(o, a, w, base, n2, base + n2, n - n2, wbase, gran));
                    new ArraysParallelSortHelpers.FJDouble.Sorter(o3, a, w, base + n2, n - n2, wbase + n2, gran).fork();
                }
                DualPivotQuicksort.sort(a, base, base + i - 1, w, wbase, i);
                this.tryComplete();
            }
        }
        
        static final class Merger extends CountedCompleter<Void>
        {
            static final long serialVersionUID = 2446542900576103244L;
            final double[] a;
            final double[] w;
            final int lbase;
            final int lsize;
            final int rbase;
            final int rsize;
            final int wbase;
            final int gran;
            
            Merger(CountedCompleter<?> completer, double[] a, double[] w, int lbase, int lsize, int rbase, int rsize, int wbase, int gran) {
                super(completer);
                this.a = a;
                this.w = w;
                this.lbase = lbase;
                this.lsize = lsize;
                this.rbase = rbase;
                this.rsize = rsize;
                this.wbase = wbase;
                this.gran = gran;
            }
            
            @Override
            public final void compute() {
                double[] a = this.a;
                double[] w = this.w;
                int lbase = this.lbase;
                int lsize = this.lsize;
                int rbase = this.rbase;
                int rsize = this.rsize;
                int wbase = this.wbase;
                int gran = this.gran;
                if (a == null || w == null || lbase < 0 || rbase < 0 || wbase < 0) {
                    throw new IllegalStateException();
                }
                while (true) {
                    int n;
                    int n3;
                    if (lsize >= rsize) {
                        if (lsize <= gran) {
                            break;
                        }
                        n3 = rsize;
                        double n5 = a[(n = lsize >>> 1) + lbase];
                        int j = 0;
                        while (j < n3) {
                            int n6 = j + n3 >>> 1;
                            if (n5 <= a[n6 + rbase]) {
                                n3 = n6;
                            }
                            else {
                                j = n6 + 1;
                            }
                        }
                    }
                    else {
                        if (rsize <= gran) {
                            break;
                        }
                        n = lsize;
                        double n2 = a[(n3 = rsize >>> 1) + rbase];
                        int i = 0;
                        while (i < n) {
                            int n4 = i + n >>> 1;
                            if (n2 <= a[n4 + lbase]) {
                                n = n4;
                            }
                            else {
                                i = n4 + 1;
                            }
                        }
                    }
                    ArraysParallelSortHelpers.FJDouble.Merger o = new ArraysParallelSortHelpers.FJDouble.Merger(this, a, w, lbase + n, lsize - n, rbase + n3, rsize - n3, wbase + n + n3, gran);
                    rsize = n3;
                    lsize = n;
                    this.addToPendingCount(1);
                    o.fork();
                }
                int n7;
                int n8;
                double n11;
                for (n7 = lbase + lsize, n8 = rbase + rsize; lbase < n7 && rbase < n8; w[wbase++] = n11) {
                    double n9 = a[lbase];
                    double n10;
                    if (n9 <= (n10 = a[rbase])) {
                        ++lbase;
                        n11 = n9;
                    }
                    else {
                        ++rbase;
                        n11 = n10;
                    }
                }
                if (rbase < n8) {
                    System.arraycopy(a, rbase, w, wbase, n8 - rbase);
                }
                else if (lbase < n7) {
                    System.arraycopy(a, lbase, w, wbase, n7 - lbase);
                }
                this.tryComplete();
                return;
            }
        }
    }
}
